@@ -18,17 +18,15 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2011 Red Hat, Inc.
+ * Copyright 2007 - 2011 Red Hat, Inc.
  */
 
-#include "config.h"
+#include "nm-default.h"
 
-#include <glib.h>
 #include <string.h>
 #include <strings.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <glib/gi18n.h>
 
 #include "crypto.h"
 
@@ -148,13 +146,6 @@ parse_old_openssl_key_file (const GByteArray *contents,
 	}
 
 	str = g_string_new_len (NULL, end - start);
-	if (!str) {
-		g_set_error (error, NM_CRYPTO_ERROR,
-		             NM_CRYPTO_ERR_OUT_OF_MEMORY,
-		             _("Not enough memory to store PEM file data."));
-		goto parse_error;
-	}
-
 	for (ln = lines; *ln; ln++) {
 		char *p = *ln;
 
@@ -213,6 +204,8 @@ parse_old_openssl_key_file (const GByteArray *contents,
 				cipher = g_strdup (p);
 			} else if (!strcasecmp (p, "DES-CBC")) {
 				cipher = g_strdup (p);
+			} else if (!strcasecmp (p, "AES-128-CBC")) {
+				cipher = g_strdup (p);
 			} else {
 				g_set_error (error, NM_CRYPTO_ERROR,
 				             NM_CRYPTO_ERR_UNKNOWN_KEY_TYPE,
@@ -308,15 +301,9 @@ parse_pkcs8_key_file (const GByteArray *contents,
 
 	if (der && length) {
 		key = g_byte_array_sized_new (length);
-		if (key) {
-			g_byte_array_append (key, der, length);
-			g_assert (key->len == length);
-			*out_encrypted = encrypted;
-		} else {
-			g_set_error_literal (error, NM_CRYPTO_ERROR,
-			                     NM_CRYPTO_ERR_OUT_OF_MEMORY,
-			                     _("Not enough memory to store private key data."));
-		}
+		g_byte_array_append (key, der, length);
+		g_assert (key->len == length);
+		*out_encrypted = encrypted;
 	} else {
 		g_set_error_literal (error, NM_CRYPTO_ERROR,
 		                     NM_CRYPTO_ERR_DECODE_FAILED,
@@ -336,14 +323,8 @@ file_to_g_byte_array (const char *filename, GError **error)
 
 	if (g_file_get_contents (filename, &contents, &length, error)) {
 		array = g_byte_array_sized_new (length);
-		if (array) {
-			g_byte_array_append (array, (guint8 *) contents, length);
-			g_assert (array->len == length);
-		} else {
-			g_set_error (error, NM_CRYPTO_ERROR,
-				         NM_CRYPTO_ERR_OUT_OF_MEMORY,
-				         _("Not enough memory to store certificate data."));
-		}
+		g_byte_array_append (array, (guint8 *) contents, length);
+		g_assert (array->len == length);
 		g_free (contents);
 	}
 	return array;
@@ -374,17 +355,11 @@ convert_iv (const char *src,
 
 	num /= 2;
 	c = g_malloc0 (num + 1);
-	if (c == NULL) {
-		g_set_error (error, NM_CRYPTO_ERROR,
-		             NM_CRYPTO_ERR_OUT_OF_MEMORY,
-		             _("Not enough memory to store the IV."));
-        return NULL;
-	}
 
 	conv[2] = '\0';
 	for (i = 0; i < num; i++) {
-        conv[0] = src[(i * 2)];
-        conv[1] = src[(i * 2) + 1];
+		conv[0] = src[(i * 2)];
+		conv[1] = src[(i * 2) + 1];
 		if (!g_ascii_isxdigit (conv[0]) || !g_ascii_isxdigit (conv[1])) {
 			g_set_error (error, NM_CRYPTO_ERROR,
 			             NM_CRYPTO_ERR_RAW_IV_INVALID,
@@ -403,12 +378,12 @@ error:
 }
 
 static char *
-make_des_key (const char *cipher,
-              const char *salt,
-              const gsize salt_len,
-              const char *password,
-              gsize *out_len,
-              GError **error)
+make_des_aes_key (const char *cipher,
+                  const char *salt,
+                  const gsize salt_len,
+                  const char *password,
+                  gsize *out_len,
+                  GError **error)
 {
 	char *key;
 	guint32 digest_len;
@@ -423,6 +398,8 @@ make_des_key (const char *cipher,
 		digest_len = 24;
 	else if (!strcmp (cipher, "DES-CBC"))
 		digest_len = 8;
+	else if (!strcmp (cipher, "AES-128-CBC"))
+		digest_len = 16;
 	else {
 		g_set_error (error, NM_CRYPTO_ERROR,
 		             NM_CRYPTO_ERR_UNKNOWN_CIPHER,
@@ -431,13 +408,10 @@ make_des_key (const char *cipher,
 		return NULL;
 	}
 
-	key = g_malloc0 (digest_len + 1);
-	if (!key) {
-		g_set_error (error, NM_CRYPTO_ERROR,
-		             NM_CRYPTO_ERR_OUT_OF_MEMORY,
-		             _("Not enough memory to decrypt private key."));
+	if (password[0] == '\0')
 		return NULL;
-	}
+
+	key = g_malloc0 (digest_len + 1);
 
 	if (!crypto_md5_hash (salt,
 	                      salt_len,
@@ -482,8 +456,8 @@ decrypt_key (const char *cipher,
 	if (!bin_iv)
 		return NULL;
 
-	/* Convert the PIN and IV into a DES key */
-	key = make_des_key (cipher, bin_iv, bin_iv_len, password, &key_len, error);
+	/* Convert the password and IV into a DES or AES key */
+	key = make_des_aes_key (cipher, bin_iv, bin_iv_len, password, &key_len, error);
 	if (!key || !key_len)
 		goto out;
 
@@ -495,13 +469,7 @@ decrypt_key (const char *cipher,
 	                         error);
 	if (output && decrypted_len) {
 		decrypted = g_byte_array_sized_new (decrypted_len);
-		if (decrypted)
-			g_byte_array_append (decrypted, (guint8 *) output, decrypted_len);
-		else {
-			g_set_error (error, NM_CRYPTO_ERROR,
-					     NM_CRYPTO_ERR_OUT_OF_MEMORY,
-					     _("Not enough memory to store decrypted private key."));
-		}
+		g_byte_array_append (decrypted, (guint8 *) output, decrypted_len);
 	}
 
 out:
@@ -556,11 +524,11 @@ crypto_decrypt_private_key_data (const GByteArray *contents,
 
 		if (password) {
 			decrypted = decrypt_key (cipher,
-						             key_type,
-						             data,
-						             iv,
-						             password,
-						             error);
+			                         key_type,
+			                         data,
+			                         iv,
+			                         password,
+			                         error);
 		}
 		g_byte_array_free (data, TRUE);
 	}
@@ -599,18 +567,18 @@ extract_pem_cert_data (GByteArray *contents, GError **error)
 
 	if (!find_tag (PEM_CERT_BEGIN, contents, 0, &start)) {
 		g_set_error (error, NM_CRYPTO_ERROR,
-			         NM_CRYPTO_ERR_FILE_FORMAT_INVALID,
-			         _("PEM certificate had no start tag '%s'."),
-			         PEM_CERT_BEGIN);
+		             NM_CRYPTO_ERR_FILE_FORMAT_INVALID,
+		             _("PEM certificate had no start tag '%s'."),
+		             PEM_CERT_BEGIN);
 		goto done;
 	}
 
 	start += strlen (PEM_CERT_BEGIN);
 	if (!find_tag (PEM_CERT_END, contents, start, &end)) {
 		g_set_error (error, NM_CRYPTO_ERROR,
-			         NM_CRYPTO_ERR_FILE_FORMAT_INVALID,
-			         _("PEM certificate had no end tag '%s'."),
-			         PEM_CERT_END);
+		             NM_CRYPTO_ERR_FILE_FORMAT_INVALID,
+		             _("PEM certificate had no end tag '%s'."),
+		             PEM_CERT_END);
 		goto done;
 	}
 
@@ -622,18 +590,12 @@ extract_pem_cert_data (GByteArray *contents, GError **error)
 
 	if (der && length) {
 		cert = g_byte_array_sized_new (length);
-		if (cert) {
-			g_byte_array_append (cert, der, length);
-			g_assert (cert->len == length);
-		} else {
-			g_set_error (error, NM_CRYPTO_ERROR,
-						 NM_CRYPTO_ERR_OUT_OF_MEMORY,
-						 _("Not enough memory to store certificate data."));
-		}
+		g_byte_array_append (cert, der, length);
+		g_assert (cert->len == length);
 	} else {
 		g_set_error (error, NM_CRYPTO_ERROR,
-			         NM_CRYPTO_ERR_DECODE_FAILED,
-			         _("Failed to decode certificate."));
+		             NM_CRYPTO_ERR_DECODE_FAILED,
+		             _("Failed to decode certificate."));
 	}
 
 done:
@@ -783,4 +745,3 @@ crypto_verify_private_key (const char *filename,
 	}
 	return format;
 }
-
