@@ -16,11 +16,12 @@
  * Copyright (C) 2010 Red Hat, Inc.
  */
 
-#ifndef NM_DNS_PLUGIN_H
-#define NM_DNS_PLUGIN_H
+#ifndef __NETWORKMANAGER_DNS_PLUGIN_H__
+#define __NETWORKMANAGER_DNS_PLUGIN_H__
 
-#include <glib.h>
-#include <glib-object.h>
+#include "nm-default.h"
+
+#include "nm-config-data.h"
 
 #define NM_TYPE_DNS_PLUGIN            (nm_dns_plugin_get_type ())
 #define NM_DNS_PLUGIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_DNS_PLUGIN, NMDnsPlugin))
@@ -42,19 +43,20 @@ typedef struct {
 	GObjectClass parent;
 
 	/* Methods */
-	gboolean (*init) (NMDnsPlugin *self);
 
 	/* Called when DNS information is changed.  'vpn_configs' is a list of
 	 * NMIP4Config or NMIP6Config objects from VPN connections, while
 	 * 'dev_configs' is a list of NMPI4Config or NMIP6Config objects from
 	 * active devices.  'other_configs' represent other IP configuration that
-	 * may be in-use.  Configs of the same IP version are sorted in priority
+	 * may be in-use.  'global_config' is the optional global DNS
+	 * configuration.  Configs of the same IP version are sorted in priority
 	 * order.
 	 */
 	gboolean (*update) (NMDnsPlugin *self,
 	                    const GSList *vpn_configs,
 	                    const GSList *dev_configs,
 	                    const GSList *other_configs,
+	                    const NMGlobalDnsConfig *global_config,
 	                    const char *hostname);
 
 	/* Subclasses should override and return TRUE if they start a local
@@ -93,6 +95,7 @@ gboolean nm_dns_plugin_update (NMDnsPlugin *self,
                                const GSList *vpn_configs,
                                const GSList *dev_configs,
                                const GSList *other_configs,
+                               const NMGlobalDnsConfig *global_config,
                                const char *hostname);
 
 /* For subclasses/plugins */
@@ -110,5 +113,5 @@ GPid nm_dns_plugin_child_spawn (NMDnsPlugin *self,
 
 gboolean nm_dns_plugin_child_kill (NMDnsPlugin *self);
 
-#endif /* NM_DNS_PLUGIN_H */
+#endif /* __NETWORKMANAGER_DNS_PLUGIN_H__ */
 
