@@ -18,7 +18,8 @@
  * Copyright (C) 2008 - 2012 Red Hat, Inc.
  */
 
-#include <config.h>
+#include "nm-default.h"
+
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -29,20 +30,31 @@
 
 #include "nm-dnsmasq-manager.h"
 #include "nm-dnsmasq-utils.h"
-#include "nm-logging.h"
-#include "nm-glib-compat.h"
-#include "nm-posix-signals.h"
 #include "nm-utils.h"
+#include "NetworkManagerUtils.h"
+#include "nm-core-internal.h"
+
+#define _NMLOG_DOMAIN         LOGD_SHARING
+#define _NMLOG_PREFIX_NAME    "dnsmasq-manager"
+#define _NMLOG(level, ...) \
+    G_STMT_START { \
+        nm_log ((level), _NMLOG_DOMAIN, \
+                "%s" _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+                _NMLOG_PREFIX_NAME": " \
+                _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+    } G_STMT_END
 
 typedef struct {
 	char *iface;
 	char *pidfile;
 	GPid pid;
-	guint32 dm_watch_id;
+	guint dm_watch_id;
 } NMDnsMasqManagerPrivate;
 
 #define NM_DNSMASQ_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_DNSMASQ_MANAGER, NMDnsMasqManagerPrivate))
 
+#define CONFDIR NMCONFDIR "/dnsmasq-shared.d"
+
 G_DEFINE_TYPE (NMDnsMasqManager, nm_dnsmasq_manager, G_TYPE_OBJECT)
 
 enum {
@@ -53,22 +65,6 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-typedef enum {
-	NM_DNSMASQ_MANAGER_ERROR_NOT_FOUND,
-	NM_DNSMASQ_MANAGER_ERROR_INVALID_IP_RANGE,
-} NMDnsMasqManagerError;
-
-GQuark
-nm_dnsmasq_manager_error_quark (void)
-{
-	static GQuark quark;
-
-	if (!quark)
-		quark = g_quark_from_static_string ("nm_dnsmasq_manager_error");
-
-	return quark;
-}
-
 static void
 nm_dnsmasq_manager_init (NMDnsMasqManager *manager)
 {
@@ -98,14 +94,14 @@ nm_dnsmasq_manager_class_init (NMDnsMasqManagerClass *manager_class)
 
 	/* signals */
 	signals[STATE_CHANGED] =
-		g_signal_new ("state-changed",
-				    G_OBJECT_CLASS_TYPE (object_class),
-				    G_SIGNAL_RUN_FIRST,
-				    G_STRUCT_OFFSET (NMDnsMasqManagerClass, state_changed),
-				    NULL, NULL,
-				    g_cclosure_marshal_VOID__UINT,
-				    G_TYPE_NONE, 1,
-				    G_TYPE_UINT);
+	     g_signal_new (NM_DNS_MASQ_MANAGER_STATE_CHANGED,
+	                   G_OBJECT_CLASS_TYPE (object_class),
+	                   G_SIGNAL_RUN_FIRST,
+	                   G_STRUCT_OFFSET (NMDnsMasqManagerClass, state_changed),
+	                   NULL, NULL,
+	                   g_cclosure_marshal_VOID__UINT,
+	                   G_TYPE_NONE, 1,
+	                   G_TYPE_UINT);
 }
 
 NMDnsMasqManager *
@@ -118,7 +114,7 @@ nm_dnsmasq_manager_new (const char *iface)
 
 	priv = NM_DNSMASQ_MANAGER_GET_PRIVATE (manager);
 	priv->iface = g_strdup (iface);
-	priv->pidfile = g_strdup_printf (LOCALSTATEDIR "/run/nm-dnsmasq-%s.pid", iface);
+	priv->pidfile = g_strdup_printf (RUNSTATEDIR "/nm-dnsmasq-%s.pid", iface);
 
 	return manager;
 }
@@ -168,27 +164,6 @@ nm_cmd_line_add_string (NMCmdLine *cmd, const char *str)
 
 /*******************************************/
 
-static inline const char *
-nm_find_dnsmasq (void)
-{
-	static const char *dnsmasq_binary_paths[] = {
-		DNSMASQ_PATH,
-		"/usr/local/sbin/dnsmasq",
-		"/usr/sbin/dnsmasq",
-		"/sbin/dnsmasq",
-		NULL
-	};
-	const char **dnsmasq_binary = dnsmasq_binary_paths;
-
-	while (*dnsmasq_binary != NULL) {
-		if (**dnsmasq_binary && g_file_test (*dnsmasq_binary, G_FILE_TEST_EXISTS))
-			break;
-		dnsmasq_binary++;
-	}
-
-	return *dnsmasq_binary;
-}
-
 static void
 dm_exit_code (guint dm_exit_status)
 {
@@ -207,7 +182,7 @@ dm_exit_code (guint dm_exit_status)
 	case 4:
 		msg = "Memory allocation failure";
 		break;
-	case 5: 
+	case 5:
 		msg = "Other problem";
 		break;
 	default:
@@ -216,7 +191,7 @@ dm_exit_code (guint dm_exit_status)
 		break;
 	}
 
-	nm_log_warn (LOGD_SHARING, "dnsmasq exited with error: %s (%d)", msg, dm_exit_status);
+	_LOGW ("dnsmasq exited with error: %s (%d)", msg, dm_exit_status);
 }
 
 static void
@@ -231,42 +206,38 @@ dm_watch_cb (GPid pid, gint status, gpointer user_data)
 		if (err != 0)
 			dm_exit_code (err);
 	} else if (WIFSTOPPED (status)) {
-		nm_log_warn (LOGD_SHARING, "dnsmasq stopped unexpectedly with signal %d", WSTOPSIG (status));
+		_LOGW ("dnsmasq stopped unexpectedly with signal %d", WSTOPSIG (status));
 	} else if (WIFSIGNALED (status)) {
-		nm_log_warn (LOGD_SHARING, "dnsmasq died with signal %d", WTERMSIG (status));
+		_LOGW ("dnsmasq died with signal %d", WTERMSIG (status));
 	} else {
-		nm_log_warn (LOGD_SHARING, "dnsmasq died from an unknown cause");
+		_LOGW ("dnsmasq died from an unknown cause");
 	}
-  
+
 	priv->pid = 0;
+	priv->dm_watch_id = 0;
 
 	g_signal_emit (manager, signals[STATE_CHANGED], 0, NM_DNSMASQ_STATUS_DEAD);
 }
 
 static NMCmdLine *
 create_dm_cmd_line (const char *iface,
-                    NMIP4Config *ip4_config,
+                    const NMPlatformIP4Address *listen_address,
                     const char *pidfile,
                     GError **error)
 {
-	const char *dm_binary;
 	NMCmdLine *cmd;
 	GString *s;
-	const NMPlatformIP4Address *tmp;
 	char first[INET_ADDRSTRLEN];
 	char last[INET_ADDRSTRLEN];
 	char localaddr[INET_ADDRSTRLEN];
 	char *error_desc = NULL;
+	const char *dm_binary;
 
-	dm_binary = nm_find_dnsmasq ();
-	if (!dm_binary) {
-		g_set_error_literal (error, NM_DNSMASQ_MANAGER_ERROR, NM_DNSMASQ_MANAGER_ERROR_NOT_FOUND,
-		                     "Could not find dnsmasq binary.");
+	g_return_val_if_fail (listen_address, NULL);
+
+	dm_binary = nm_utils_find_helper ("dnsmasq", DNSMASQ_PATH, error);
+	if (!dm_binary)
 		return NULL;
-	}
-
-	/* Find the IP4 address to use */
-	tmp = nm_ip4_config_get_address (ip4_config, 0);
 
 	/* Create dnsmasq command line */
 	cmd = nm_cmd_line_new ();
@@ -298,19 +269,20 @@ create_dm_cmd_line (const char *iface,
 	nm_cmd_line_add_string (cmd, "--strict-order");
 
 	s = g_string_new ("--listen-address=");
-	nm_utils_inet4_ntop (tmp->address, localaddr);
+	nm_utils_inet4_ntop (listen_address->address, localaddr);
 	g_string_append (s, localaddr);
 	nm_cmd_line_add_string (cmd, s->str);
 	g_string_free (s, TRUE);
 
-	if (!nm_dnsmasq_utils_get_range (tmp, first, last, &error_desc)) {
+	if (!nm_dnsmasq_utils_get_range (listen_address, first, last, &error_desc)) {
 		g_set_error_literal (error,
-		                     NM_DNSMASQ_MANAGER_ERROR,
-		                     NM_DNSMASQ_MANAGER_ERROR_INVALID_IP_RANGE,
+		                     NM_MANAGER_ERROR,
+		                     NM_MANAGER_ERROR_FAILED,
 		                     error_desc);
-		nm_log_warn (LOGD_SHARING, "Failed to find DHCP address ranges: %s", error_desc);
+		_LOGW ("failed to find DHCP address ranges: %s", error_desc);
 		g_free (error_desc);
-		goto error;
+		nm_cmd_line_destroy (cmd);
+		return NULL;
 	}
 
 	s = g_string_new ("--dhcp-range=");
@@ -330,57 +302,49 @@ create_dm_cmd_line (const char *iface,
 	nm_cmd_line_add_string (cmd, s->str);
 	g_string_free (s, TRUE);
 
+	/* dnsmasq exits if the conf dir is not present */
+	if (g_file_test (CONFDIR, G_FILE_TEST_IS_DIR))
+		nm_cmd_line_add_string (cmd, "--conf-dir=" CONFDIR);
+
 	return cmd;
-
-error:
-	nm_cmd_line_destroy (cmd);
-	return NULL;
 }
 
 static void
-dm_child_setup (gpointer user_data G_GNUC_UNUSED)
-{
-	/* We are in the child process at this point */
-	pid_t pid = getpid ();
-	setpgid (pid, pid);
-
-	/*
-	 * We blocked signals in main(). We need to restore original signal
-	 * mask for dnsmasq here so that it can receive signals.
-	 */
-	nm_unblock_posix_signals (NULL);
-}
-
-static void
-kill_existing_for_iface (const char *iface, const char *pidfile)
+kill_existing_by_pidfile (const char *pidfile)
 {
 	char *contents = NULL;
-	glong pid;
-	char *proc_path = NULL;
+	pid_t pid;
+	char proc_path[250];
 	char *cmdline_contents = NULL;
+	guint64 start_time;
+	const char *exe;
 
-	if (!g_file_get_contents (pidfile, &contents, NULL, NULL))
+	if (   !pidfile
+	    || !g_file_get_contents (pidfile, &contents, NULL, NULL))
+		return;
+
+	pid = _nm_utils_ascii_str_to_int64 (contents, 10, 1, G_MAXUINT64, 0);
+	if (pid == 0)
 		goto out;
 
-	pid = strtol (contents, NULL, 10);
-	if (pid < 1 || pid > INT_MAX)
+	start_time = nm_utils_get_start_time_for_pid (pid, NULL, NULL);
+	if (start_time == 0)
 		goto out;
 
-	proc_path = g_strdup_printf ("/proc/%ld/cmdline", pid);
+	nm_sprintf_buf (proc_path, "/proc/%lld/cmdline", (long long) pid);
 	if (!g_file_get_contents (proc_path, &cmdline_contents, NULL, NULL))
 		goto out;
 
-	if (strstr (cmdline_contents, "bin/dnsmasq")) {
-		if (kill (pid, 0) == 0) {
-			nm_log_dbg (LOGD_SHARING, "Killing stale dnsmasq process %ld", pid);
-			kill (pid, SIGKILL);
-		}
-		unlink (pidfile);
+	exe = strrchr (cmdline_contents, '/');
+	if (   (exe && strcmp (&exe[1], "dnsmasq") == 0)
+	    || (strcmp (cmdline_contents, DNSMASQ_PATH) == 0)) {
+		nm_utils_kill_process_sync (pid, start_time, SIGKILL, LOGD_SHARING,
+		                            "dnsmasq", 0, 0, 500);
 	}
 
 out:
+	unlink (pidfile);
 	g_free (cmdline_contents);
-	g_free (proc_path);
 	g_free (contents);
 }
 
@@ -391,37 +355,34 @@ nm_dnsmasq_manager_start (NMDnsMasqManager *manager,
 {
 	NMDnsMasqManagerPrivate *priv;
 	NMCmdLine *dm_cmd;
-	char *cmd_str;
+	gs_free char *cmd_str = NULL;
 
 	g_return_val_if_fail (NM_IS_DNSMASQ_MANAGER (manager), FALSE);
-	if (error)
-		g_return_val_if_fail (*error == NULL, FALSE);
+	g_return_val_if_fail (!error || !*error, FALSE);
+	g_return_val_if_fail (nm_ip4_config_get_num_addresses (ip4_config) > 0, FALSE);
 
 	priv = NM_DNSMASQ_MANAGER_GET_PRIVATE (manager);
 
-	kill_existing_for_iface (priv->iface, priv->pidfile);
+	kill_existing_by_pidfile (priv->pidfile);
 
-	dm_cmd = create_dm_cmd_line (priv->iface, ip4_config, priv->pidfile, error);
+	dm_cmd = create_dm_cmd_line (priv->iface, nm_ip4_config_get_address (ip4_config, 0), priv->pidfile, error);
 	if (!dm_cmd)
 		return FALSE;
 
 	g_ptr_array_add (dm_cmd->array, NULL);
 
-	nm_log_info (LOGD_SHARING, "Starting dnsmasq...");
-
-	cmd_str = nm_cmd_line_to_str (dm_cmd);
-	nm_log_dbg (LOGD_SHARING, "Command line: %s", cmd_str);
-	g_free (cmd_str);
+	_LOGI ("starting dnsmasq...");
+	_LOGD ("command line: %s", (cmd_str = nm_cmd_line_to_str (dm_cmd)));
 
 	priv->pid = 0;
 	if (!g_spawn_async (NULL, (char **) dm_cmd->array->pdata, NULL,
-					G_SPAWN_DO_NOT_REAP_CHILD,
-					dm_child_setup,
-					NULL, &priv->pid, error)) {
+	                    G_SPAWN_DO_NOT_REAP_CHILD,
+	                    nm_utils_setpgid, NULL,
+	                    &priv->pid, error)) {
 		goto out;
 	}
 
-	nm_log_dbg (LOGD_SHARING, "dnsmasq started with pid %d", priv->pid);
+	_LOGD ("dnsmasq started with pid %d", priv->pid);
 
 	priv->dm_watch_id = g_child_watch_add (priv->pid, (GChildWatchFunc) dm_watch_cb, manager);
 
@@ -432,22 +393,6 @@ nm_dnsmasq_manager_start (NMDnsMasqManager *manager,
 	return priv->pid > 0;
 }
 
-static gboolean
-ensure_killed (gpointer data)
-{
-	int pid = GPOINTER_TO_INT (data);
-
-	if (kill (pid, 0) == 0)
-		kill (pid, SIGKILL);
-
-	/* ensure the child is reaped */
-	nm_log_dbg (LOGD_SHARING, "waiting for dnsmasq pid %d to exit", pid);
-	waitpid (pid, NULL, 0);
-	nm_log_dbg (LOGD_SHARING, "dnsmasq pid %d cleaned up", pid);
-
-	return FALSE;
-}
-
 void
 nm_dnsmasq_manager_stop (NMDnsMasqManager *manager)
 {
@@ -457,23 +402,10 @@ nm_dnsmasq_manager_stop (NMDnsMasqManager *manager)
 
 	priv = NM_DNSMASQ_MANAGER_GET_PRIVATE (manager);
 
-	if (priv->dm_watch_id) {
-		g_source_remove (priv->dm_watch_id);
-		priv->dm_watch_id = 0;
-	}
+	nm_clear_g_source (&priv->dm_watch_id);
 
 	if (priv->pid) {
-		if (kill (priv->pid, SIGTERM) == 0)
-			g_timeout_add_seconds (2, ensure_killed, GINT_TO_POINTER (priv->pid));
-		else {
-			kill (priv->pid, SIGKILL);
-
-			/* ensure the child is reaped */
-			nm_log_dbg (LOGD_SHARING, "waiting for dnsmasq pid %d to exit", priv->pid);
-			waitpid (priv->pid, NULL, 0);
-			nm_log_dbg (LOGD_SHARING, "dnsmasq pid %d cleaned up", priv->pid);
-		}
-
+		nm_utils_kill_child_async (priv->pid, SIGTERM, LOGD_SHARING, "dnsmasq", 2000, NULL, NULL);
 		priv->pid = 0;
 	}
 
