@@ -13,23 +13,21 @@
  * Copyright (C) 2012 Red Hat, Inc.
  */
 
-#ifndef NM_CONNECTION_PROVIDER_H
-#define NM_CONNECTION_PROVIDER_H
+#ifndef __NETWORKMANAGER_CONNECTION_PROVIDER_H__
+#define __NETWORKMANAGER_CONNECTION_PROVIDER_H__
 
-#include <glib-object.h>
 #include <nm-connection.h>
 
-#define NM_TYPE_CONNECTION_PROVIDER      (nm_connection_provider_get_type ())
-#define NM_CONNECTION_PROVIDER(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_CONNECTION_PROVIDER, NMConnectionProvider))
-#define NM_IS_CONNECTION_PROVIDER(obj)   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NM_TYPE_CONNECTION_PROVIDER))
-#define NM_CONNECTION_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), NM_TYPE_CONNECTION_PROVIDER, NMConnectionProvider))
+#include "nm-default.h"
 
-typedef struct _NMConnectionProvider NMConnectionProvider;
+#define NM_TYPE_CONNECTION_PROVIDER               (nm_connection_provider_get_type ())
+#define NM_CONNECTION_PROVIDER(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_CONNECTION_PROVIDER, NMConnectionProvider))
+#define NM_IS_CONNECTION_PROVIDER(obj)            (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NM_TYPE_CONNECTION_PROVIDER))
+#define NM_CONNECTION_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), NM_TYPE_CONNECTION_PROVIDER, NMConnectionProviderInterface))
 
 #define NM_CP_SIGNAL_CONNECTION_ADDED        "cp-connection-added"
 #define NM_CP_SIGNAL_CONNECTION_UPDATED      "cp-connection-updated"
 #define NM_CP_SIGNAL_CONNECTION_REMOVED      "cp-connection-removed"
-#define NM_CP_SIGNAL_CONNECTIONS_LOADED      "cp-connections-loaded"
 
 
 /**
@@ -45,7 +43,7 @@ typedef gboolean (*NMConnectionFilterFunc) (NMConnectionProvider *provider,
                                             gpointer func_data);
 
 
-struct _NMConnectionProvider {
+typedef struct {
 	GTypeInterface g_iface;
 
 	/* Methods */
@@ -58,12 +56,14 @@ struct _NMConnectionProvider {
 
 	const GSList * (*get_connections) (NMConnectionProvider *self);
 
-	gboolean (*has_connections_loaded) (NMConnectionProvider *self);
-
 	NMConnection * (*add_connection) (NMConnectionProvider *self,
 	                                  NMConnection *connection,
+	                                  gboolean save_to_disk,
 	                                  GError **error);
 
+	NMConnection * (*get_connection_by_uuid) (NMConnectionProvider *self,
+	                                          const char *uuid);
+
 	/* Signals */
 	void (*connection_added)   (NMConnectionProvider *self, NMConnection *connection);
 
@@ -71,11 +71,17 @@ struct _NMConnectionProvider {
 
 	void (*connection_removed) (NMConnectionProvider *self, NMConnection *connection);
 
-	void (*connections_loaded) (NMConnectionProvider *self);
-};
+} NMConnectionProviderInterface;
 
 GType nm_connection_provider_get_type (void);
 
+/**
+ * nm_connection_provider_get:
+ *
+ * Returns: the global #NMConnectionProvider
+ */
+NMConnectionProvider *nm_connection_provider_get (void);
+
 /**
  * nm_connection_provider_get_best_connections:
  * @self: the #NMConnectionProvider
@@ -109,29 +115,21 @@ GSList *nm_connection_provider_get_best_connections (NMConnectionProvider *self,
  */
 const GSList *nm_connection_provider_get_connections (NMConnectionProvider *self);
 
-/**
- * nm_connection_provider_has_connections_loaded:
- * @self: the #NMConnectionProvider
- *
- * Returns: TRUE or FALSE indicating whether the connections of the provider are already
- *   loaded. If they are not yet loaded, the provider will not emit the signals
- *   NM_CP_SIGNAL_CONNECTION_ADDED, NM_CP_SIGNAL_CONNECTION_UPDATED and
- *   NM_CP_SIGNAL_CONNECTION_REMOVED until NM_CP_SIGNAL_CONNECTIONS_LOADED gets
- *   emited.
- */
-gboolean nm_connection_provider_has_connections_loaded (NMConnectionProvider *self);
-
-
 /**
  * nm_connection_provider_add_connection:
  * @self: the #NMConnectionProvider
  * @connection: the connection to be added
+ * @save_to_disk: whether to store the connection on disk
  * @error: returns any error if adding fails
  *
  * returns: a newly added #NMConnection.
  */
 NMConnection *nm_connection_provider_add_connection (NMConnectionProvider *self,
                                                      NMConnection *connection,
+                                                     gboolean save_to_disk,
                                                      GError **error);
 
-#endif /* NM_CONNECTION_PROVIDER_H */
+NMConnection *nm_connection_provider_get_connection_by_uuid (NMConnectionProvider *self,
+                                                             const char *uuid);
+
+#endif /* __NETWORKMANAGER_CONNECTION_PROVIDER_H__ */
