@@ -18,16 +18,14 @@
  * Copyright (C) 2015 Red Hat, Inc.
  */
 
-#include <string.h>
+#include "nm-default.h"
 
-#include "config.h"
+#include <string.h>
 
 #include "nm-route-manager.h"
 #include "nm-platform.h"
 #include "nmp-object.h"
 #include "nm-core-internal.h"
-#include "nm-logging.h"
-#include "gsystem-local-alloc.h"
 #include "NetworkManagerUtils.h"
 
 /* if within half a second after adding an IP address a matching device-route shows
@@ -75,6 +73,10 @@ typedef struct {
 
 G_DEFINE_TYPE (NMRouteManager, nm_route_manager, G_TYPE_OBJECT);
 
+NM_GOBJECT_PROPERTIES_DEFINE_BASE (
+	PROP_PLATFORM,
+);
+
 NM_DEFINE_SINGLETON_GETTER (NMRouteManager, nm_route_manager_get, NM_TYPE_ROUTE_MANAGER);
 
 /*********************************************************************************************/
@@ -112,28 +114,9 @@ static const VTableIP vtable_v4, vtable_v6;
 
 /*********************************************************************************************/
 
-#define _LOG_PREFIX_NAME "route-mgr"
-
-#define _LOG(level, addr_family, ...) \
-    G_STMT_START { \
-        const int __addr_family = (addr_family); \
-        const NMLogLevel __level = (level); \
-        const NMLogDomain __domain = __addr_family == AF_INET ? LOGD_IP4 : (__addr_family == AF_INET6 ? LOGD_IP6 : LOGD_IP); \
-        \
-        if (nm_logging_enabled (__level, __domain)) { \
-            char __ch = __addr_family == AF_INET ? '4' : (__addr_family == AF_INET6 ? '6' : '-'); \
-            char __prefix[30] = _LOG_PREFIX_NAME; \
-            \
-            if ((self) != singleton_instance) \
-                g_snprintf (__prefix, sizeof (__prefix), "%s%c[%p]", _LOG_PREFIX_NAME, __ch, (self)); \
-            else \
-                __prefix[STRLEN (_LOG_PREFIX_NAME)] = __ch; \
-            _nm_log ((level), (__domain), 0, \
-                     "%s: " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
-                     __prefix _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
-        } \
-    } G_STMT_END
-#define _LOG_LEVEL_ENABLED(level, addr_family) \
+#define _NMLOG_PREFIX_NAME   "route-mgr"
+#undef  _NMLOG_ENABLED
+#define _NMLOG_ENABLED(level, addr_family) \
     ({ \
         const int __addr_family = (addr_family); \
         const NMLogLevel __level = (level); \
@@ -141,19 +124,25 @@ static const VTableIP vtable_v4, vtable_v6;
         \
         nm_logging_enabled (__level, __domain); \
     })
-
-#ifdef NM_MORE_LOGGING
-#define _LOGT_ENABLED(addr_family)   _LOG_LEVEL_ENABLED (LOGL_TRACE, addr_family)
-#define _LOGT(addr_family, ...)      _LOG (LOGL_TRACE, addr_family, __VA_ARGS__)
-#else
-#define _LOGT_ENABLED(addr_family)   (FALSE && _LOG_LEVEL_ENABLED (LOGL_TRACE, addr_family))
-#define _LOGT(addr_family, ...)      G_STMT_START { if (FALSE) { _LOG (LOGL_TRACE, addr_family, __VA_ARGS__); } } G_STMT_END
-#endif
-
-#define _LOGD(addr_family, ...)      _LOG (LOGL_DEBUG, addr_family, __VA_ARGS__)
-#define _LOGI(addr_family, ...)      _LOG (LOGL_INFO , addr_family, __VA_ARGS__)
-#define _LOGW(addr_family, ...)      _LOG (LOGL_WARN , addr_family, __VA_ARGS__)
-#define _LOGE(addr_family, ...)      _LOG (LOGL_ERR  , addr_family, __VA_ARGS__)
+#define _NMLOG(level, addr_family, ...) \
+    G_STMT_START { \
+        const int __addr_family = (addr_family); \
+        const NMLogLevel __level = (level); \
+        const NMLogDomain __domain = __addr_family == AF_INET ? LOGD_IP4 : (__addr_family == AF_INET6 ? LOGD_IP6 : LOGD_IP); \
+        \
+        if (nm_logging_enabled (__level, __domain)) { \
+            char __ch = __addr_family == AF_INET ? '4' : (__addr_family == AF_INET6 ? '6' : '-'); \
+            char __prefix[30] = _NMLOG_PREFIX_NAME; \
+            \
+            if ((self) != singleton_instance) \
+                g_snprintf (__prefix, sizeof (__prefix), "%s%c[%p]", _NMLOG_PREFIX_NAME, __ch, (self)); \
+            else \
+                __prefix[NM_STRLEN (_NMLOG_PREFIX_NAME)] = __ch; \
+            _nm_log ((level), (__domain), 0, \
+                     "%s: " _NM_UTILS_MACRO_FIRST(__VA_ARGS__), \
+                     __prefix _NM_UTILS_MACRO_REST(__VA_ARGS__)); \
+        } \
+    } G_STMT_END
 
 /*********************************************************************************************/
 
@@ -161,7 +150,7 @@ static gboolean _ip4_device_routes_cancel (NMRouteManager *self);
 
 /*********************************************************************************************/
 
-#if defined (NM_MORE_ASSERTS) && !defined (G_DISABLE_ASSERT)
+#if NM_MORE_ASSERTS && !defined (G_DISABLE_ASSERT)
 inline static void
 ASSERT_route_index_valid (const VTableIP *vtable, const GArray *entries, const RouteIndex *index, gboolean unique_ifindexes)
 {
@@ -478,15 +467,15 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 	ASSERT_route_index_valid (vtable, known_routes, known_routes_idx, FALSE);
 
 	_LOGD (vtable->vt->addr_family, "%3d: sync %u IPv%c routes", ifindex, known_routes_idx->len, vtable->vt->is_ip4 ? '4' : '6');
-	if (_LOGT_ENABLED (vtable->vt->addr_family)) {
+	if (_LOGt_ENABLED (vtable->vt->addr_family)) {
 		for (i = 0; i < known_routes_idx->len; i++) {
-			_LOGT (vtable->vt->addr_family, "%3d: sync new route #%u: %s",
-			       ifindex, i, vtable->vt->route_to_string (VTABLE_ROUTE_INDEX (vtable, known_routes, i)));
+			_LOGt (vtable->vt->addr_family, "%3d: sync new route #%u: %s",
+			       ifindex, i, vtable->vt->route_to_string (VTABLE_ROUTE_INDEX (vtable, known_routes, i), NULL, 0));
 		}
 		for (i = 0; i < ipx_routes->index->len; i++)
-			_LOGT (vtable->vt->addr_family, "%3d: STATE: has     #%u - %s (%lld)",
+			_LOGt (vtable->vt->addr_family, "%3d: STATE: has     #%u - %s (%lld)",
 			       ifindex, i,
-			       vtable->vt->route_to_string (ipx_routes->index->entries[i]),
+			       vtable->vt->route_to_string (ipx_routes->index->entries[i], NULL, 0),
 			       (long long) g_array_index (ipx_routes->effective_metrics, gint64, i));
 	}
 
@@ -526,7 +515,8 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 				cur_ipx_route->rx.ifindex = ifindex;
 				cur_ipx_route->rx.metric = vtable->vt->metric_normalize (cur_ipx_route->rx.metric);
 				ipx_routes_changed = TRUE;
-				_LOGT (vtable->vt->addr_family, "%3d: STATE: update  #%u - %s", ifindex, i_ipx_routes, vtable->vt->route_to_string (cur_ipx_route));
+				_LOGt (vtable->vt->addr_family, "%3d: STATE: update  #%u - %s", ifindex, i_ipx_routes,
+				       vtable->vt->route_to_string (cur_ipx_route, NULL, 0));
 			}
 		} else if (cur_known_route) {
 			g_assert (!cur_ipx_route || route_id_cmp_result > 0);
@@ -584,7 +574,8 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 			    && cur_plat_route->rx.metric == *p_effective_metric) {
 				/* we are about to delete cur_ipx_route and we have a matching route
 				 * in platform. Delete it. */
-				_LOGT (vtable->vt->addr_family, "%3d: platform rt-rm #%u - %s", ifindex, i_plat_routes, vtable->vt->route_to_string (cur_plat_route));
+				_LOGt (vtable->vt->addr_family, "%3d: platform rt-rm #%u - %s", ifindex, i_plat_routes,
+				       vtable->vt->route_to_string (cur_plat_route, NULL, 0));
 				vtable->vt->route_delete (priv->platform, ifindex, cur_plat_route);
 			}
 		}
@@ -596,7 +587,8 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 			for (i = 0; i < to_delete_indexes->len; i++) {
 				guint idx = g_array_index (to_delete_indexes, guint, i);
 
-				_LOGT (vtable->vt->addr_family, "%3d: STATE: delete  #%u - %s", ifindex, idx, vtable->vt->route_to_string (ipx_routes->index->entries[idx]));
+				_LOGt (vtable->vt->addr_family, "%3d: STATE: delete  #%u - %s", ifindex, idx,
+				       vtable->vt->route_to_string (ipx_routes->index->entries[idx], NULL, 0));
 				g_array_index (to_delete_indexes, guint, i) = _route_index_reverse_idx (vtable, ipx_routes->index, idx, ipx_routes->entries);
 			}
 			g_array_sort (to_delete_indexes, (GCompareFunc) _sort_indexes_cmp);
@@ -620,7 +612,8 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 
 				g_array_index (ipx_routes->effective_metrics_reverse, gint64, j++) = -1;
 
-				_LOGT (vtable->vt->addr_family, "%3d: STATE: added   #%u - %s", ifindex, ipx_routes->entries->len - 1, vtable->vt->route_to_string (ipx_route));
+				_LOGt (vtable->vt->addr_family, "%3d: STATE: added   #%u - %s", ifindex, ipx_routes->entries->len - 1,
+				       vtable->vt->route_to_string (ipx_route, NULL, 0));
 			}
 			g_ptr_array_unref (to_add_routes);
 		}
@@ -643,7 +636,7 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 		 * causes the route on the unmanaged device to be replaced).
 		 * Still, that is not much different then from messing with unmanaged routes when
 		 * the effective and the intended metrics equal. The rules is: NM will leave routes
-		 * on unmanged devices alone, unless they conflict with what NM wants to configure.
+		 * on unmanaged devices alone, unless they conflict with what NM wants to configure.
 		 ***************************************************************************/
 
 		g_array_set_size (ipx_routes->effective_metrics, ipx_routes->entries->len);
@@ -705,9 +698,9 @@ _vx_route_sync (const VTableIP *vtable, NMRouteManager *self, int ifindex, const
 				break;
 			}
 next:
-			_LOGT (vtable->vt->addr_family, "%3d: new metric     #%u - %s (%lld)",
+			_LOGt (vtable->vt->addr_family, "%3d: new metric     #%u - %s (%lld)",
 			       ifindex, i_ipx_routes,
-			       vtable->vt->route_to_string (cur_ipx_route),
+			       vtable->vt->route_to_string (cur_ipx_route, NULL, 0),
 			       (long long) *p_effective_metric);
 		}
 	}
@@ -730,7 +723,7 @@ next:
 
 			g_assert (cur_plat_route->rx.ifindex == ifindex);
 
-			_LOGT (vtable->vt->addr_family, "%3d: platform rt    #%u - %s", ifindex, i_plat_routes, vtable->vt->route_to_string (cur_plat_route));
+			_LOGt (vtable->vt->addr_family, "%3d: platform rt    #%u - %s", ifindex, i_plat_routes, vtable->vt->route_to_string (cur_plat_route, NULL, 0));
 
 			/* skip over @cur_ipx_route that are ordered before @cur_plat_route */
 			while (   cur_ipx_route
@@ -884,7 +877,7 @@ next:
 						_LOGD (vtable->vt->addr_family,
 						       "ignore error adding IPv%c route to kernel: %s",
 						       vtable->vt->is_ip4 ? '4' : '6',
-						       vtable->vt->route_to_string (cur_ipx_route));
+						       vtable->vt->route_to_string (cur_ipx_route, NULL, 0));
 					} else {
 						/* Remember that there was a failure, but for now continue trying
 						 * to sync the remaining routes. */
@@ -951,8 +944,11 @@ nm_route_manager_ip6_route_sync (NMRouteManager *self, int ifindex, const GArray
 gboolean
 nm_route_manager_route_flush (NMRouteManager *self, int ifindex)
 {
-	return    nm_route_manager_ip4_route_sync (self, ifindex, NULL, FALSE, TRUE)
-	       && nm_route_manager_ip6_route_sync (self, ifindex, NULL, FALSE, TRUE);
+	bool success = TRUE;
+
+	success &= (bool) nm_route_manager_ip4_route_sync (self, ifindex, NULL, FALSE, TRUE);
+	success &= (bool) nm_route_manager_ip6_route_sync (self, ifindex, NULL, FALSE, TRUE);
+	return success;
 }
 
 /*********************************************************************************************/
@@ -1000,7 +996,7 @@ _ip4_device_routes_idle_cb (IP4DeviceRoutePurgeEntry *entry)
 		return G_SOURCE_REMOVE;
 	}
 
-	_LOGT (vtable_v4.vt->addr_family, "device-route: delete %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
+	_LOGt (vtable_v4.vt->addr_family, "device-route: delete %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
 
 	nm_platform_ip4_route_delete (priv->platform,
 	                              entry->obj->ip4_route.ifindex,
@@ -1019,7 +1015,6 @@ _ip4_device_routes_ip4_route_changed (NMPlatform *platform,
                                       int ifindex,
                                       const NMPlatformIP4Route *route,
                                       NMPlatformSignalChangeType change_type,
-                                      NMPlatformReason reason,
                                       NMRouteManager *self)
 {
 	NMRouteManagerPrivate *priv;
@@ -1043,14 +1038,14 @@ _ip4_device_routes_ip4_route_changed (NMPlatform *platform,
 		return;
 
 	if (_ip4_device_routes_entry_expired (entry, nm_utils_get_monotonic_timestamp_ns ())) {
-		_LOGT (vtable_v4.vt->addr_family, "device-route: cleanup-ch %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
+		_LOGt (vtable_v4.vt->addr_family, "device-route: cleanup-ch %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
 		g_hash_table_remove (priv->ip4_device_routes.entries, entry->obj);
 		_ip4_device_routes_cancel (self);
 		return;
 	}
 
 	if (entry->idle_id == 0) {
-		_LOGT (vtable_v4.vt->addr_family, "device-route: schedule %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
+		_LOGt (vtable_v4.vt->addr_family, "device-route: schedule %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
 		entry->idle_id = g_idle_add ((GSourceFunc) _ip4_device_routes_idle_cb, entry);
 	}
 }
@@ -1063,7 +1058,7 @@ _ip4_device_routes_cancel (NMRouteManager *self)
 	if (priv->ip4_device_routes.gc_id) {
 		if (g_hash_table_size (priv->ip4_device_routes.entries) > 0)
 			return G_SOURCE_CONTINUE;
-		_LOGT (vtable_v4.vt->addr_family, "device-route: cancel");
+		_LOGt (vtable_v4.vt->addr_family, "device-route: cancel");
 		if (priv->platform)
 			g_signal_handlers_disconnect_by_func (priv->platform, G_CALLBACK (_ip4_device_routes_ip4_route_changed), self);
 		nm_clear_g_source (&priv->ip4_device_routes.gc_id);
@@ -1084,7 +1079,7 @@ _ip4_device_routes_gc (NMRouteManager *self)
 	g_hash_table_iter_init (&iter, priv->ip4_device_routes.entries);
 	while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &entry)) {
 		if (_ip4_device_routes_entry_expired (entry, now)) {
-			_LOGT (vtable_v4.vt->addr_family, "device-route: cleanup-gc %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
+			_LOGt (vtable_v4.vt->addr_family, "device-route: cleanup-gc %s", nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
 			g_hash_table_iter_remove (&iter);
 		}
 	}
@@ -1117,7 +1112,7 @@ nm_route_manager_ip4_route_register_device_route_purge_list (NMRouteManager *sel
 		IP4DeviceRoutePurgeEntry *entry;
 
 		entry = _ip4_device_routes_purge_entry_create (self, &g_array_index (device_route_purge_list, NMPlatformIP4Route, i), now_ns);
-		_LOGT (vtable_v4.vt->addr_family, "device-route: watch (%s) %s",
+		_LOGt (vtable_v4.vt->addr_family, "device-route: watch (%s) %s",
 		                                  g_hash_table_contains (priv->ip4_device_routes.entries, entry->obj)
 		                                      ? "update" : "new",
 		                                  nmp_object_to_string (entry->obj, NMP_OBJECT_TO_STRING_PUBLIC, NULL, 0));
@@ -1147,13 +1142,32 @@ static const VTableIP vtable_v6 = {
 
 /*********************************************************************************************/
 
+static void
+set_property (GObject *object, guint prop_id,
+              const GValue *value, GParamSpec *pspec)
+{
+	NMRouteManager *self = NM_ROUTE_MANAGER (object);
+	NMRouteManagerPrivate *priv = NM_ROUTE_MANAGER_GET_PRIVATE (self);
+
+	switch (prop_id) {
+	case PROP_PLATFORM:
+		/* construct-only */
+		priv->platform = g_value_get_object (value) ? : NM_PLATFORM_GET;
+		if (!priv->platform)
+			g_return_if_reached ();
+		g_object_ref (priv->platform);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 nm_route_manager_init (NMRouteManager *self)
 {
 	NMRouteManagerPrivate *priv = NM_ROUTE_MANAGER_GET_PRIVATE (self);
 
-	priv->platform = g_object_ref (NM_PLATFORM_GET);
-
 	priv->ip4_routes.entries = g_array_new (FALSE, FALSE, sizeof (NMPlatformIP4Route));
 	priv->ip6_routes.entries = g_array_new (FALSE, FALSE, sizeof (NMPlatformIP6Route));
 	priv->ip4_routes.effective_metrics = g_array_new (FALSE, FALSE, sizeof (gint64));
@@ -1168,6 +1182,14 @@ nm_route_manager_init (NMRouteManager *self)
 	                                                         (GDestroyNotify) _ip4_device_routes_purge_entry_free);
 }
 
+NMRouteManager *
+nm_route_manager_new (NMPlatform *platform)
+{
+	return g_object_new (NM_TYPE_ROUTE_MANAGER,
+	                     NM_ROUTE_MANAGER_PLATFORM, platform,
+	                     NULL);
+}
+
 static void
 dispose (GObject *object)
 {
@@ -1177,8 +1199,6 @@ dispose (GObject *object)
 	g_hash_table_remove_all (priv->ip4_device_routes.entries);
 	_ip4_device_routes_cancel (self);
 
-	g_clear_object (&priv->platform);
-
 	G_OBJECT_CLASS (nm_route_manager_parent_class)->dispose (object);
 }
 
@@ -1198,6 +1218,8 @@ finalize (GObject *object)
 
 	g_hash_table_unref (priv->ip4_device_routes.entries);
 
+	g_clear_object (&priv->platform);
+
 	G_OBJECT_CLASS (nm_route_manager_parent_class)->finalize (object);
 }
 
@@ -1209,6 +1231,15 @@ nm_route_manager_class_init (NMRouteManagerClass *klass)
 	g_type_class_add_private (klass, sizeof (NMRouteManagerPrivate));
 
 	/* virtual methods */
+	object_class->set_property = set_property;
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
+
+	obj_properties[PROP_PLATFORM] =
+	    g_param_spec_object (NM_ROUTE_MANAGER_PLATFORM, "", "",
+	                         NM_TYPE_PLATFORM,
+	                         G_PARAM_WRITABLE |
+	                         G_PARAM_CONSTRUCT_ONLY |
+	                         G_PARAM_STATIC_STRINGS);
+	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 }
