@@ -18,12 +18,11 @@
  *
  */
 
-#include <glib.h>
+#include "nm-default.h"
+
 #include <string.h>
 
-#include <nm-utils.h>
-
-#include "nm-logging.h"
+#include "nm-core-internal.h"
 #include "interface_parser.h"
 #include "parser.h"
 
@@ -460,176 +459,52 @@ test17_read_static_ipv4 (const char *path)
 {
 	NMConnection *connection;
 	NMSettingConnection *s_con;
-	NMSettingIP4Config *s_ip4;
+	NMSettingIPConfig *s_ip4;
 	NMSettingWired *s_wired;
-	char *unmanaged = NULL;
 	GError *error = NULL;
-	const char* tmp;
-	const char *expected_address = "10.0.0.3";
-	const char *expected_id = "Ifupdown (eth0)";
-	const char *expected_dns1 = "10.0.0.1";
-	const char *expected_dns2 = "10.0.0.2";
-	const char *expected_search1 = "example.com";
-	const char *expected_search2 = "foo.example.com";
-	guint32 expected_prefix = 8;
-	NMIP4Address *ip4_addr;
-	guint32 addr;
-#define TEST17_NAME "wired-static-verify-ip4"
+	gboolean success;
+	NMIPAddress *ip4_addr;
 	if_block *block = NULL;
 
-	const char* file = "test17-" TEST17_NAME;
-
-	init_ifparser_with_file (path, file);
+	init_ifparser_with_file (path, "test17-wired-static-verify-ip4");
 	block = ifparser_getfirst ();
-	connection = nm_connection_new();
+	connection = nm_simple_connection_new();
+	g_assert (connection);
+
 	ifupdown_update_connection_from_if_block(connection, block, &error);
+	g_assert_no_error (error);
 
-	ASSERT (connection != NULL,
-			TEST17_NAME, "failed to read %s: %s", file, error->message);
-
-	ASSERT (nm_connection_verify (connection, &error),
-			TEST17_NAME, "failed to verify %s: %s", file, error->message);
-
-	ASSERT (unmanaged == NULL,
-			TEST17_NAME, "failed to verify %s: unexpected unmanaged value", file);
+	success = nm_connection_verify (connection, &error);
+	g_assert_no_error (error);
+	g_assert (success);
 
 	/* ===== CONNECTION SETTING ===== */
-
 	s_con = nm_connection_get_setting_connection (connection);
-	ASSERT (s_con != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME);
-
-	/* ID */
-	tmp = nm_setting_connection_get_id (s_con);
-	ASSERT (tmp != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s / %s key",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME,
-			NM_SETTING_CONNECTION_ID);
-	ASSERT (strcmp (tmp, expected_id) == 0,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value: %s",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME,
-			NM_SETTING_CONNECTION_ID, tmp);
+	g_assert (s_con);
+	g_assert_cmpstr (nm_setting_connection_get_id (s_con), ==, "Ifupdown (eth0)");
 
 	/* ===== WIRED SETTING ===== */
-
 	s_wired = nm_connection_get_setting_wired (connection);
-	ASSERT (s_wired != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_WIRED_SETTING_NAME);
+	g_assert (s_wired);
 
 	/* ===== IPv4 SETTING ===== */
-
-	ASSERT (inet_pton (AF_INET, expected_address, &addr) > 0,
-			TEST17_NAME, "failed to verify %s: couldn't convert IP address #1",
-			file);
-
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
-	ASSERT (s_ip4 != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME);
+	g_assert (s_ip4);
+	g_assert_cmpstr (nm_setting_ip_config_get_method (s_ip4), ==, NM_SETTING_IP4_CONFIG_METHOD_MANUAL);
 
-	/* Method */
-	tmp = nm_setting_ip4_config_get_method (s_ip4);
-	ASSERT (strcmp (tmp, NM_SETTING_IP4_CONFIG_METHOD_MANUAL) == 0,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_METHOD);
+	g_assert_cmpint (nm_setting_ip_config_get_num_addresses (s_ip4), ==, 1);
+	ip4_addr = nm_setting_ip_config_get_address (s_ip4, 0);
+	g_assert (ip4_addr != NULL);
+	g_assert_cmpstr (nm_ip_address_get_address (ip4_addr), ==, "10.0.0.3");
+	g_assert_cmpint (nm_ip_address_get_prefix (ip4_addr), ==, 8);
 
-	/* IP addresses */
-	ASSERT (nm_setting_ip4_config_get_num_addresses (s_ip4) == 1,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_ADDRESSES);
+	g_assert_cmpint (nm_setting_ip_config_get_num_dns (s_ip4), ==, 2);
+	g_assert_cmpstr (nm_setting_ip_config_get_dns (s_ip4, 0), ==, "10.0.0.1");
+	g_assert_cmpstr (nm_setting_ip_config_get_dns (s_ip4, 1), ==, "10.0.0.2");
 
-	ip4_addr = nm_setting_ip4_config_get_address (s_ip4, 0);
-	ASSERT (ip4_addr,
-			TEST17_NAME, "failed to verify %s: missing IP4 address #1",
-			file);
-
-	ASSERT (nm_ip4_address_get_prefix (ip4_addr) == expected_prefix,
-			TEST17_NAME, "failed to verify %s: unexpected IP4 address prefix",
-			file);
-
-	ASSERT (nm_ip4_address_get_address (ip4_addr) == addr,
-			TEST17_NAME, "failed to verify %s: unexpected IP4 address: %s",
-			file, addr);
-
-	/* DNS Addresses */
-	ASSERT (nm_setting_ip4_config_get_num_dns (s_ip4) == 2,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	ASSERT (inet_pton (AF_INET, expected_dns1, &addr) > 0,
-			TEST17_NAME, "failed to verify %s: couldn't convert DNS IP address #1",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	ASSERT (nm_setting_ip4_config_get_dns (s_ip4, 0) == addr,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value #1",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	ASSERT (inet_pton (AF_INET, expected_dns2, &addr) > 0,
-			TEST17_NAME, "failed to verify %s: couldn't convert DNS IP address #2",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	ASSERT (nm_setting_ip4_config_get_dns (s_ip4, 1) == addr,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value #2",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	ASSERT (nm_setting_ip4_config_get_num_addresses (s_ip4) == 1,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	/* DNS search domains */
-	ASSERT (nm_setting_ip4_config_get_num_dns_searches (s_ip4) == 2,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS);
-
-	tmp = nm_setting_ip4_config_get_dns_search (s_ip4, 0);
-	ASSERT (tmp != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s / %s key",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS_SEARCH);
-	ASSERT (strcmp (tmp, expected_search1) == 0,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS_SEARCH);
-
-	tmp = nm_setting_ip4_config_get_dns_search (s_ip4, 1);
-	ASSERT (tmp != NULL,
-			TEST17_NAME, "failed to verify %s: missing %s / %s key",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS_SEARCH);
-
-	ASSERT (strcmp (tmp, expected_search2) == 0,
-			TEST17_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_DNS_SEARCH);
+	g_assert_cmpint (nm_setting_ip_config_get_num_dns_searches (s_ip4), ==, 2);
+	g_assert_cmpstr (nm_setting_ip_config_get_dns_search (s_ip4, 0), ==, "example.com");
+	g_assert_cmpstr (nm_setting_ip_config_get_dns_search (s_ip4, 1), ==, "foo.example.com");
 
 	g_object_unref (connection);
 }
@@ -639,199 +514,52 @@ test18_read_static_ipv6 (const char *path)
 {
 	NMConnection *connection;
 	NMSettingConnection *s_con;
-	NMSettingIP6Config *s_ip6;
+	NMSettingIPConfig *s_ip6;
 	NMSettingWired *s_wired;
-	char *unmanaged = NULL;
 	GError *error = NULL;
-	const char* tmp;
-	const char *expected_address = "fc00::1";
-	const char *expected_id = "Ifupdown (myip6tunnel)";
-	const char *expected_dns1 = "fc00::2";
-	const char *expected_dns2 = "fc00::3";
-	const char *expected_search1 = "example.com";
-	const char *expected_search2 = "foo.example.com";
-	guint32 expected_prefix = 64;
-	NMIP6Address *ip6_addr;
-	struct in6_addr addr;
+	gboolean success;
+	NMIPAddress *ip6_addr;
 	if_block *block = NULL;
-#define TEST18_NAME "wired-static-verify-ip6"
-	const char* file = "test18-" TEST18_NAME;
 
-	init_ifparser_with_file (path, file);
+	init_ifparser_with_file (path, "test18-wired-static-verify-ip6");
 	block = ifparser_getfirst ();
-	connection = nm_connection_new();
+	connection = nm_simple_connection_new();
+	g_assert (connection);
 	ifupdown_update_connection_from_if_block(connection, block, &error);
+	g_assert_no_error (error);
 
-	ASSERT (connection != NULL,
-			TEST18_NAME
-			"failed to read %s: %s", file, error->message);
-
-	ASSERT (nm_connection_verify (connection, &error),
-			TEST18_NAME,
-			"failed to verify %s: %s", file, error->message);
-
-	ASSERT (unmanaged == NULL,
-			TEST18_NAME,
-			"failed to verify %s: unexpected unmanaged value", file);
+	success = nm_connection_verify (connection, &error);
+	g_assert_no_error (error);
+	g_assert (success);
 
 	/* ===== CONNECTION SETTING ===== */
-
 	s_con = nm_connection_get_setting_connection (connection);
-	ASSERT (s_con != NULL,
-			TEST18_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME);
-
-	/* ID */
-	tmp = nm_setting_connection_get_id (s_con);
-	ASSERT (tmp != NULL,
-			TEST18_NAME,
-			"failed to verify %s: missing %s / %s key",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME,
-			NM_SETTING_CONNECTION_ID);
-
-	ASSERT (strcmp (tmp, expected_id) == 0,
-			TEST18_NAME,
-			"failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_CONNECTION_SETTING_NAME,
-			NM_SETTING_CONNECTION_ID);
+	g_assert (s_con);
+	g_assert_cmpstr (nm_setting_connection_get_id (s_con), ==, "Ifupdown (myip6tunnel)");
 
 	/* ===== WIRED SETTING ===== */
-
 	s_wired = nm_connection_get_setting_wired (connection);
-	ASSERT (s_wired != NULL,
-			TEST18_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_WIRED_SETTING_NAME);
+	g_assert (s_wired);
 
 	/* ===== IPv6 SETTING ===== */
-
-	ASSERT (inet_pton (AF_INET6, expected_address, &addr) > 0,
-			TEST18_NAME,
-			"failed to verify %s: couldn't convert IP address #1",
-			file);
-
 	s_ip6 = nm_connection_get_setting_ip6_config (connection);
-	ASSERT (s_ip6 != NULL,
-			TEST18_NAME,
-			"failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME);
+	g_assert (s_ip6);
+	g_assert_cmpstr (nm_setting_ip_config_get_method (s_ip6), ==, NM_SETTING_IP6_CONFIG_METHOD_MANUAL);
 
-	/* Method */
-	tmp = nm_setting_ip6_config_get_method (s_ip6);
-	ASSERT (strcmp (tmp, NM_SETTING_IP6_CONFIG_METHOD_MANUAL) == 0,
-			TEST18_NAME,
-			"failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_METHOD);
+	g_assert_cmpint (nm_setting_ip_config_get_num_addresses (s_ip6), ==, 1);
+	ip6_addr = nm_setting_ip_config_get_address (s_ip6, 0);
+	g_assert (ip6_addr != NULL);
+	g_assert_cmpstr (nm_ip_address_get_address (ip6_addr), ==, "fc00::1");
+	g_assert_cmpint (nm_ip_address_get_prefix (ip6_addr), ==, 64);
 
-	/* IP addresses */
-	ASSERT (nm_setting_ip6_config_get_num_addresses (s_ip6) == 1,
-			TEST18_NAME,
-			"failed to verify %s: unexpected number of %s / %s",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_ADDRESSES);
+	g_assert_cmpint (nm_setting_ip_config_get_num_dns (s_ip6), ==, 2);
+	g_assert_cmpstr (nm_setting_ip_config_get_dns (s_ip6, 0), ==, "fc00::2");
+	g_assert_cmpstr (nm_setting_ip_config_get_dns (s_ip6, 1), ==, "fc00::3");
 
-	ip6_addr = nm_setting_ip6_config_get_address (s_ip6, 0);
-	ASSERT (ip6_addr,
-			TEST18_NAME,
-			"failed to verify %s: missing %s / %s #1",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_ADDRESSES);
+	g_assert_cmpint (nm_setting_ip_config_get_num_dns_searches (s_ip6), ==, 2);
+	g_assert_cmpstr (nm_setting_ip_config_get_dns_search (s_ip6, 0), ==, "example.com");
+	g_assert_cmpstr (nm_setting_ip_config_get_dns_search (s_ip6, 1), ==, "foo.example.com");
 
-	ASSERT (nm_ip6_address_get_prefix (ip6_addr) == expected_prefix,
-			TEST18_NAME
-			"failed to verify %s: unexpected %s / %s prefix",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_ADDRESSES);
-
-	ASSERT (IN6_ARE_ADDR_EQUAL (nm_ip6_address_get_address (ip6_addr),
-								&addr),
-			TEST18_NAME,
-			"failed to verify %s: unexpected %s / %s",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_ADDRESSES);
-
-	/* DNS Addresses */
-	ASSERT (nm_setting_ip6_config_get_num_dns (s_ip6) == 2,
-			TEST18_NAME,
-			"failed to verify %s: unexpected number of %s / %s values",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS);
-
-	ASSERT (inet_pton (AF_INET6, expected_dns1, &addr) > 0,
-			TEST18_NAME,
-			"failed to verify %s: couldn't convert DNS IP address #1",
-			file);
-
-	ASSERT (IN6_ARE_ADDR_EQUAL (nm_setting_ip6_config_get_dns (s_ip6, 0),
-								&addr),
-			TEST18_NAME,
-			"failed to verify %s: unexpected %s / %s #1",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS);
-
-	ASSERT (inet_pton (AF_INET6, expected_dns2, &addr) > 0,
-			TEST18_NAME,
-			"failed to verify %s: couldn't convert DNS IP address #2",
-			file);
-
-	ASSERT (IN6_ARE_ADDR_EQUAL (nm_setting_ip6_config_get_dns (s_ip6, 1),
-								&addr),
-			TEST18_NAME, "failed to verify %s: unexpected %s / %s #2",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS);
-
-	/* DNS search domains */
-	ASSERT (nm_setting_ip6_config_get_num_dns_searches (s_ip6) == 2,
-			TEST18_NAME,
-			"failed to verify %s: unexpected number of %s / %s values",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS_SEARCH);
-
-	tmp = nm_setting_ip6_config_get_dns_search (s_ip6, 0);
-	ASSERT (tmp != NULL,
-			"wired-ipv6-manual-verify-ip6",
-			"failed to verify %s: missing %s / %s #1",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS_SEARCH);
-
-	ASSERT (strcmp (tmp, expected_search1) == 0,
-			"wired-ipv6-manual-verify-ip6",
-			"failed to verify %s: unexpected %s / %s #1",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS_SEARCH);
-
-	tmp = nm_setting_ip6_config_get_dns_search (s_ip6, 1);
-	ASSERT (tmp != NULL,
-			TEST18_NAME,
-			"failed to verify %s: missing %s / %s #2",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS_SEARCH);
-
-	ASSERT (strcmp (tmp, expected_search2) == 0,
-			TEST18_NAME,
-			"failed to verify %s: unexpected %s / %s #2",
-			file,
-			NM_SETTING_IP6_CONFIG_SETTING_NAME,
-			NM_SETTING_IP6_CONFIG_DNS_SEARCH);
-
-	g_free (unmanaged);
 	g_object_unref (connection);
 }
 
@@ -839,63 +567,32 @@ static void
 test19_read_static_ipv4_plen (const char *path)
 {
 	NMConnection *connection;
-	NMSettingIP4Config *s_ip4;
-	char *unmanaged = NULL;
+	NMSettingIPConfig *s_ip4;
 	GError *error = NULL;
-	const char *expected_address = "10.0.0.3";
-	guint32 expected_prefix = 8;
-	NMIP4Address *ip4_addr;
-	guint32 addr;
-#define TEST19_NAME "wired-static-verify-ip4-plen"
+	NMIPAddress *ip4_addr;
 	if_block *block = NULL;
+	gboolean success;
 
-	const char* file = "test19-" TEST19_NAME;
-
-	init_ifparser_with_file (path, file);
+	init_ifparser_with_file (path, "test19-wired-static-verify-ip4-plen");
 	block = ifparser_getfirst ();
-	connection = nm_connection_new();
+	connection = nm_simple_connection_new();
+	g_assert (connection);
 	ifupdown_update_connection_from_if_block(connection, block, &error);
+	g_assert_no_error (error);
 
-	ASSERT (connection != NULL,
-			TEST19_NAME, "failed to read %s: %s", file, error->message);
-
-	ASSERT (nm_connection_verify (connection, &error),
-			TEST19_NAME, "failed to verify %s: %s", file, error->message);
-
-	ASSERT (unmanaged == NULL,
-			TEST19_NAME, "failed to verify %s: unexpected unmanaged value", file);
+	success = nm_connection_verify (connection, &error);
+	g_assert_no_error (error);
+	g_assert (success);
 
 	/* ===== IPv4 SETTING ===== */
-
-	ASSERT (inet_pton (AF_INET, expected_address, &addr) > 0,
-			TEST19_NAME, "failed to verify %s: couldn't convert IP address #1",
-			file);
-
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
-	ASSERT (s_ip4 != NULL,
-			TEST19_NAME, "failed to verify %s: missing %s setting",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME);
+	g_assert (s_ip4);
 
-	/* IP addresses */
-	ASSERT (nm_setting_ip4_config_get_num_addresses (s_ip4) == 1,
-			TEST19_NAME, "failed to verify %s: unexpected %s / %s key value",
-			file,
-			NM_SETTING_IP4_CONFIG_SETTING_NAME,
-			NM_SETTING_IP4_CONFIG_ADDRESSES);
-
-	ip4_addr = nm_setting_ip4_config_get_address (s_ip4, 0);
-	ASSERT (ip4_addr,
-			TEST19_NAME, "failed to verify %s: missing IP4 address #1",
-			file);
-
-	ASSERT (nm_ip4_address_get_prefix (ip4_addr) == expected_prefix,
-			TEST19_NAME, "failed to verify %s: unexpected IP4 address prefix",
-			file);
-
-	ASSERT (nm_ip4_address_get_address (ip4_addr) == addr,
-			TEST19_NAME, "failed to verify %s: unexpected IP4 address: %s",
-			file, addr);
+	g_assert_cmpint (nm_setting_ip_config_get_num_addresses (s_ip4), ==, 1);
+	ip4_addr = nm_setting_ip_config_get_address (s_ip4, 0);
+	g_assert (ip4_addr != NULL);
+	g_assert_cmpstr (nm_ip_address_get_address (ip4_addr), ==, "10.0.0.3");
+	g_assert_cmpint (nm_ip_address_get_prefix (ip4_addr), ==, 8);
 
 	g_object_unref (connection);
 }
@@ -927,20 +624,12 @@ test20_source_stanza (const char *path)
 	expected_free (e);
 }
 
+NMTST_DEFINE ();
+
 int
 main (int argc, char **argv)
 {
-	GError *error = NULL;
-
-#if !GLIB_CHECK_VERSION (2, 35, 0)
-	g_type_init ();
-#endif
-
-	if (!nm_utils_init (&error))
-		FAIL ("nm-utils-init", "failed to initialize libnm-util: %s", error->message);
-	nm_logging_setup ("WARN", "DEFAULT", NULL, NULL);
-
-	g_test_init (&argc, &argv, NULL);
+	nmtst_init_assert_logging (&argc, &argv, "WARN", "DEFAULT");
 
 	if (0)
 		dump_blocks ();
